package somlib.som;

import java.io.*;

public class Mapping
/*
 * with this application vectors can be mapped on a previously trained map
 */
{

  public static void main(String[] args) throws SOMException {
    if (args.length%2 == 1)
      usage();
    else {
      int param=0;
      String mapdescr=null;
      String url_mapped_input_vec=null;
      String url_mapped_input_vec_descr=null;
      String filename=null;
      String url_weight_vec=null;
      String url_quanterr_map=null;
      String url_unit_descr=null;
      String mappingFile = null;
      boolean clearQuant=true;
      int verbose=1;

      while(param < args.length) {      //getting the command-line parameters
        if((args[param].length()>2) || (args[param].charAt(0)!='-'))
          usage();

        switch(args[param].charAt(1)) {
          case 'm':
            mapdescr = new String(args[++param]);
            break;
          case 'd':
            url_mapped_input_vec_descr = new String(args[++param]);
            break;
          case 'i':
            url_mapped_input_vec = new String(args[++param]);
            break;
          case 'f':
            filename = new String(args[++param]);
            break;
          case 'c':
            url_weight_vec = new String(args[++param]);
            break;
          case 'p':
            url_quanterr_map = new String(args[++param]);
            break;
          case 'r':
            url_unit_descr = new String(args[++param]);
            break;
          case 'n':
            mappingFile = new String(args[++param]);
            break;
          case 'q':
            if(args[++param].charAt(0) == 'f')
              clearQuant=false;
            break;
          case 'v':
            verbose = Integer.parseInt(args[++param]);
            break;
          default:
            usage();
        }
        param++;
      }

      if(mapdescr==null || url_mapped_input_vec==null)
        usage();

      Mapping mapping = new Mapping(mapdescr, clearQuant, null, (byte)verbose);
      mapping.map(url_mapped_input_vec, url_mapped_input_vec_descr);
      mapping.writeMapDescription(filename, url_weight_vec, url_quanterr_map, url_unit_descr, mappingFile, null);
    }
  }

  static void usage() {
    System.out.println("Usage: somlib.som.Mapping -m <SOM-Map-Description_in> -i <SOM-Input-Vector_in> [-d <SOM-Input-Vector-Description_in>] [-f <SOM-Map-Description_out> -c <SOM-Weight-Vector_out> -r <SOM-Unit-Description_out>] [-p <SOM-Quantisation-Error-Map_out>] [-n <SOM-Mapping-File_out>] [-v <verbosity>]");

    System.exit(1);
  }


  private somlib.textrepresentation.Logfile log = null;
  private byte verbose = 2;

  private Map map;                                                                         // the trained map
  private somlib.som.descriptions.Map demap;                       // the description of the map to be used
  private java.util.Vector url_mapped_input_vec;                 // the vectors to be mapped
  private java.util.Vector url_mapped_input_vec_descr;   // and the descriptions of the vectors to be mapped

  public Mapping(String mapdescr, boolean clearQuant, somlib.textrepresentation.Logfile log, byte verbose) throws SOMException {
    this.log=log;
    this.verbose=verbose;
    try {
      this.demap = new somlib.som.descriptions.Map(mapdescr);
    } catch(somlib.som.descriptions.DescriptionException e) {
      throw new SOMException("Mapping.<init>: couldn't read description: "+e.getMessage());
    }

       /* if a new map-topology is implemented it has to be loaded here! */
    if(demap.type.parameter.equals("rect")) {
      try {
        map = new EuclideanMap(demap.url_weight_vec.parameter, demap.url_quanterr_map.parameter, clearQuant, demap.url_unit_descr.parameter, log, verbose);
      } catch(SOMException e) {
        throw new SOMException("Mapping.<init>: couldn't initialize map: "+e.getMessage());
      }
    } else
      throw new SOMException("Map for '"+demap.type.parameter+"'-type not implemented.");

    this.url_mapped_input_vec=new java.util.Vector();
    if(demap.url_mapped_input_vec.parameter!=null) {
      for(int i=0; i<demap.url_mapped_input_vec.parameter.length; i++)
        this.url_mapped_input_vec.add(new String(demap.url_mapped_input_vec.parameter[i]));
    }
    this.url_mapped_input_vec_descr=new java.util.Vector();
    if(demap.url_mapped_input_vec_descr.parameter!=null) {
      for(int i=0; i<demap.url_mapped_input_vec_descr.parameter.length; i++)
        this.url_mapped_input_vec_descr.add(new String(demap.url_mapped_input_vec_descr.parameter[i]));
    }

    if(log!=null)
      log.append("Start: Mapping.");
    if (verbose > 0) System.out.println("  Mapping:");
  }

  public void map(String url_mapped_input_vec, String url_mapped_input_vec_descr) throws SOMException
  /*
   * the vectors @url_mapped_input_vec are mapped on the previously trained map
   */
  {
    somlib.som.descriptions.VectorIn vi = null;
    this.url_mapped_input_vec.add(new String(url_mapped_input_vec));
    if(url_mapped_input_vec_descr != null)
      this.url_mapped_input_vec_descr.add(new String(url_mapped_input_vec_descr));
    try {
      vi = new somlib.som.descriptions.VectorIn(new String(url_mapped_input_vec));
    } catch(SOMException e) {
      throw new SOMException("Mapping.map: couldn't load Vector Description: "+e.getMessage());
    }
    if(vi.vec_dim!=map.vectorDimension)
      throw new SOMException("The dimensions of the node-vectors and the input-vectors don't match.");

    if (verbose > 0) System.out.println("    ... vectors from file "+url_mapped_input_vec);
    try {
      for(int i=0; i<vi.XDim*vi.YDim; i++)
        map.map(new Vector(vi.vecs[i], vi.labels[i]));
    } catch(SOMException e) {
      throw new SOMException("Mapping.map: "+e.getMessage());
    }
  }

  public void writeMapDescription(String filename, String url_weight_vec, String url_quanterr_map, String url_unit_descr, String mappingFile, String description) throws SOMException
  /*
   * a SOM Map Description File is written with the current values
   */
  {
    if (verbose > 0) System.out.println("    writing map-description");

    if(filename!=null && filename.indexOf(File.separator) >= 0) {
      (new File(filename)).getParentFile().mkdirs();
      demap.filename = filename;
    }
    if(url_weight_vec!=null && url_weight_vec.indexOf(File.separator) >= 0) {
      (new File(url_weight_vec)).getParentFile().mkdirs();
      demap.url_weight_vec=new somlib.som.descriptions.StringParameter(url_weight_vec, demap.url_weight_vec.comment);
    }
    if(url_quanterr_map!=null && url_quanterr_map.indexOf(File.separator) >= 0) {
      (new File(url_quanterr_map)).getParentFile().mkdirs();
      demap.url_quanterr_map=new somlib.som.descriptions.StringParameter(url_quanterr_map, demap.url_quanterr_map.comment);
    }
    if(url_unit_descr!=null && url_unit_descr.indexOf(File.separator) >= 0) {
      (new File(url_unit_descr)).getParentFile().mkdirs();
      demap.url_unit_descr=new somlib.som.descriptions.StringParameter(url_unit_descr, demap.url_unit_descr.comment);
    }
    demap.quanterror_map=new somlib.som.descriptions.FloatParameter(map.getTotalQuantizationError(), demap.quanterror_map.comment);
    if(!url_mapped_input_vec.isEmpty())
      demap.url_mapped_input_vec= new somlib.som.descriptions.ListStringParameter((String[])url_mapped_input_vec.toArray(new String[0]), demap.url_mapped_input_vec.comment);
    if(!url_mapped_input_vec_descr.isEmpty())
      demap.url_mapped_input_vec_descr=new somlib.som.descriptions.ListStringParameter((String[])url_mapped_input_vec_descr.toArray(new String[0]), demap.url_mapped_input_vec_descr.comment);
    if(description!=null)
      demap.description=new somlib.som.descriptions.StringParameter(description, demap.description.comment);

    try {
      if (verbose > 0) System.out.println("      Map: writing SOM Map Description File");
      demap.writeDescr(demap.filename);
      if(url_weight_vec != null)
        map.storeToWgtFile(url_weight_vec);
      map.storeNodes(demap.url_unit_descr.parameter);
      if(demap.url_quanterr_map.parameter != null)
        map.storeQuantToErrFile(demap.url_quanterr_map.parameter);
      if(mappingFile != null)
        map.writeMappingFile(mappingFile);
    } catch(SOMException e) {
      throw new SOMException("Mapping.writeMapDescription: "+e.getMessage());
    }
  }

}
