package somlib.som;

import java.io.*;
import java.util.*;

public class Training
/*
 * with this application a new SOM can be trained with given training vectors
 */
{

  public static void main(String[] args) throws SOMException {
    if (args.length%2 == 1)
      usage();
    else {
      int param=0;
      somlib.som.functions.Functions fcts = null;
      Map map = null;
      Training trainer  = null;
      int XDim=-1, YDim=-1, iter=-1;
      float eps0=2, delta0=2;
      long seed=-1;
      String tin=null, tinde=null;
      String filename=null, url_weight_vec=null, url_unit_descr=null, trainingFile=null;
      String learnrate = new String("ELearn");
      String neighborhood = new String("ENeighbor");
      int verbose=1;

      while(param < args.length) {      //getting the command-line parameters
        if((args[param].length()>2) || (args[param].charAt(0)!='-'))
          usage();

        switch(args[param].charAt(1)) {
          case 't':
            tin = new String(args[++param]);
            break;
          case 'd':
            tinde = new String(args[++param]);
            break;
          case 'i':
            iter = Integer.parseInt(args[++param]);
            break;
          case 's':    //seed for map - init
            seed = Long.parseLong(args[++param]);
            break;
          case 'n':   //init neighborhood
            delta0 = Float.parseFloat(args[++param]);
            break;
          case 'g':   //neighborhood function
            neighborhood = new String(args[++param]);
            break;
          case 'l':   //init learnrate
            eps0 = Float.parseFloat(args[++param]);
            break;
          case 'h':  //learnrate function
            learnrate = new String(args[++param]);
            break;
          case 'x':
            XDim = Integer.parseInt(args[++param]);
            break;
          case 'y':
            YDim = Integer.parseInt(args[++param]);
            break;
          case 'f':
            filename = new String(args[++param]);
            break;
          case 'u':
            trainingFile = new String(args[++param]);
            break;
          case 'c':
            url_weight_vec = new String(args[++param]);
            break;
          case 'r':
            url_unit_descr = new String(args[++param]);
            break;
          case 'v':
            verbose = Integer.parseInt(args[++param]);
            break;
          default:
            usage();
        }
        param++;
      }

      if(XDim==-1 || YDim==-1 || iter==-1 || tin==null || filename==null)
        usage();

      try {
        fcts = new somlib.som.functions.Functions(XDim, YDim, eps0, learnrate, delta0, neighborhood, null, (byte)verbose);
      } catch(somlib.som.functions.FunctionsException e) {
        throw new SOMException("Couldn't initialize functions: "+e.getMessage());
      }
      try {
        trainer = new Training(fcts, tin, tinde, null, (byte)verbose);
        map = new EuclideanMap(XDim, YDim, trainer.vectorDimension(), seed, null, (byte)verbose);
        trainer.loadMap(map);
      } catch(SOMException e) {
        throw new SOMException("Couldn't initialize Map: "+e.getMessage());
      }

      try {
        trainer.train(iter, seed, trainingFile);
        trainer.writeMapDescription(filename, url_weight_vec, url_unit_descr, null);
      } catch(SOMException e) {
        throw new SOMException("Couldn't execute Training: "+e.getMessage());
      }
    }
  }

  static void usage() {
    System.out.println("Usage: somlib.som.Training -t <SOM-Input-Vector_in> [-d <SOM-Input-Vector-Description_in>] -i <iterations> -n <init neighborhood> [-g <neighborhood name>] -l <init learnrate> [-h <learnrate name>] -x <XDim> -y <YDim> -f <SOM-Map-Description_out> -c <SOM-Weight-Vector_out> -r <SOM-Unit-Description_out> [-s <seed for map-init>] [-u <SOM-Training-File_out>] [-v <verbosity>]");

    System.exit(1);
  }


  private somlib.textrepresentation.Logfile log = null;
  private byte verbose = 2;

  private Map map;                                              // the map to be trained
  private somlib.som.functions.Functions fcts;  // and the functions to be used for the training
  int iter=0;                                                            // training iterations
  int tt=0;                                                               // training time
  private int vecs;                                                // vector - count
  private int dims;                                                // vectors dimension
  private Vector[] iv=null;                                  // Vectors themselves
  String url_training_vec=null;                         // the *.in - file containing the Vectors
  String url_training_vec_descr=null;             // the Vector description file - doesn't need to be issued

  public Training(somlib.som.functions.Functions fcts, String url_training_vec, String url_training_vec_descr, somlib.textrepresentation.Logfile log, byte verbose) throws SOMException {
    this.log=log;
    this.verbose=verbose;
    this.fcts=fcts;
    this.url_training_vec=url_training_vec;
    this.url_training_vec_descr=url_training_vec_descr;

    if (verbose > 0) System.out.println("  Training:");
    initIVs();
    if(iv==null)
      throw new SOMException("Training: no Training-vectors found.");
    if(log!=null)
      log.append("Start: Training initialized.");
  }

  public Training(somlib.som.functions.Functions fcts, Map map, String url_training_vec, String url_training_vec_descr, somlib.textrepresentation.Logfile log, byte verbose) throws SOMException {
    this(fcts, url_training_vec, url_training_vec_descr, log, verbose);
    this.loadMap(map);
  }

  public void loadMap(Map map) throws SOMException
  /*
   * a Training session can be given a new map - an empty or previously trained one
   */
  {
    if (map==null)
      throw new SOMException("Training: no Map was given.");
    if(map.vectorDimension != dims)
      throw new SOMException("Training: The dimensions of the vectors (map and training) don't match.");
    this.map=map;
  }

  private void initIVs() throws SOMException
  /*
   * a new set of Training-vectors is initialized from the file @url_training_vec
   */
  {
    somlib.som.descriptions.VectorIn vi = null;
    try {
      vi = new somlib.som.descriptions.VectorIn(this.url_training_vec);
    } catch(somlib.som.descriptions.DescriptionException e) {
      throw new SOMException("Training.initIVs: couldn't read *.in - file: "+e.getMessage());
    }
    this.vecs=vi.XDim * vi.YDim;
    this.dims=vi.vec_dim;
    iv = new Vector[vecs];
    for(int i=0; i<vecs; i++)
      iv[i]=new Vector(vi.vecs[i], vi.labels[i]);
  }

  public void train(int iter, long seed, String trainingFile) throws SOMException
  /*
   * @iter times  a random Vector of the set of Training-Vectors is taken
   * and the map is adapted according to the functions used
   */
  {
    somlib.som.descriptions.Training training = null;
    if(trainingFile!=null)
      training = new somlib.som.descriptions.Training(map.XDim, map.YDim, dims, iter);
    this.iter=iter;
    Date start=new Date();
    int t, x, y, vecnr;
    Coordinate winnerAt, coord = new Coordinate();
    Vector anyVec, curVec;
    float factor;
    if(seed < 0)
      seed = (new Date()).getTime();
    java.util.Random rnd = new java.util.Random(seed);

    if(map==null)
      throw new SOMException("No map to be trained given!");
    if (verbose > 0) System.out.println("    Training started...\n      "+iter+" iterations");

    for(t=0; t<iter; t++) {
      vecnr = (int) (rnd.nextInt(vecs));
      anyVec=iv[vecnr];
      try {
        winnerAt=map.winnerAt(anyVec);
      } catch(SOMException e) {
        throw new SOMException("Training.train: "+e.getMessage());
      }
      if (verbose > 1) System.out.println("        "+t+".  Mapped Vector #"+vecnr+" named "+anyVec.getLabel()+" to "+winnerAt.x+"/"+winnerAt.y);
      if(training!=null) {
        String anyVecName;
        if(anyVec.label==null)
          anyVecName = String.valueOf(vecnr);
        else
          anyVecName = anyVec.getLabel();
        training.addItem(new somlib.som.descriptions.TrainingItem(t, winnerAt.x, winnerAt.y, anyVecName, map.hasDistanceToWinner(anyVec)) );
      }

      for(x=0; x<map.XDim; x++) {
        for(y=0; y<map.YDim; y++) {
          coord.x=x;
          coord.y=y;
          try {
            curVec=map.vectorAt(coord);
            factor= fcts.eps(t) * fcts.hci(t,  map.dist_ci(winnerAt, x, y) );
            map.adapt(coord,  Vector.add(curVec, Vector.sub(anyVec, curVec).mult(factor)));
          } catch(SOMException e) {
            throw new SOMException("Training.train: "+e.getMessage());
          }
        }
      }
    }
    tt = (int)(((new Date()).getTime() - start.getTime()) / 1000);
    if (verbose > 0) System.out.println("    Training finished in "+tt+"s.");
    if(training!=null) training.writeDescr(trainingFile);
  }

  public int vectorDimension()
  /*
   * the vectors dimension  (the training vectors and of the map)
   */
  {
    return dims;
  }

  public void writeMapDescription(String filename, String url_weight_vec, String url_unit_descr, String description) throws SOMException
  /*
   * a SOM Map Description File is written with the current values
   */
  {
    somlib.som.descriptions.Map demap = null;
    if(filename==null)
      throw new SOMException("No filename for mapdescription given.");
    if(filename.indexOf(File.separator) >= 0)
      (new File(filename)).getParentFile().mkdirs();

    try {
      demap = new somlib.som.descriptions.Map(map.getTopology(), map.XDim, map.YDim, map.vectorDimension);
    } catch(SOMException e) {
      throw new SOMException("Training.writeMapDescription: "+e.getMessage());
    }

    java.util.Date currentTime = new java.util.Date();
    java.text.SimpleDateFormat formatter = new java.text.SimpleDateFormat ("dd.MM.yyyy");
    demap.storage_date=new somlib.som.descriptions.StringParameter(formatter.format(currentTime));
    formatter = new java.text.SimpleDateFormat ("hh:mm:ss");
    demap.storage_time=new somlib.som.descriptions.StringParameter(formatter.format(currentTime));
//    GregorianCalendar gc = new GregorianCalendar();
//    demap.stoage_time=cg.get(Calendar.HOUR)+" "+cg.get(Calendar.MINUTE)+" "+cg.get(Calendar.SECOND)+" "+cg.get(Calendar.AM_PM);
    demap.training_time=new somlib.som.descriptions.IntegerParameter(tt, null);
    demap.learnrate_type=new somlib.som.descriptions.StringParameter(fcts.learnrateType(), null);
    demap.learnrate_init=new somlib.som.descriptions.FloatParameter(fcts.learnrateInit(), null);
    demap.neighborhood_type=new somlib.som.descriptions.StringParameter(fcts.neighborhoodType(), null);
    demap.neighborhood_init=new somlib.som.descriptions.FloatParameter(fcts.neighborhoodInit(), null);
    demap.rand_init=new somlib.som.descriptions.LongParameter(map.seed, null);
    demap.iterations_total=new somlib.som.descriptions.IntegerParameter(iter, null);
    demap.nr_trainvec_total=new somlib.som.descriptions.IntegerParameter(vecs, null);
    demap.vec_normalized=new somlib.som.descriptions.IntegerParameter(0, null);
    demap.url_training_vec=new somlib.som.descriptions.StringParameter(this.url_training_vec, null);
    demap.url_training_vec_descr=new somlib.som.descriptions.StringParameter(this.url_training_vec_descr, null);
    demap.url_weight_vec=new somlib.som.descriptions.StringParameter(url_weight_vec, null);
//    demap.quanterror_map=map.getTotalQuantizationError();
//    demap.url_quanterr_map=url_quanterr_map;
//    demap.url_mapped_input_vec=url_mapped_input_vec;   ... written by SOM mapping program
//    demap.url_mapped_input_vec_descr=url_mapped_input_vec_descr;
    demap.url_unit_descr=new somlib.som.descriptions.StringParameter(url_unit_descr, null);
    demap.description=new somlib.som.descriptions.StringParameter(description, null);

    try {
      map.storeToWgtFile(url_weight_vec);
//      map.storeQuantToErrFile(url_quanterr_map);        ... as above
      map.storeNodes(url_unit_descr);
      demap.writeDescr(filename);
    } catch(SOMException e) {
      throw new SOMException("Training.writeMapDescription: "+e.getMessage());
    }
  }

}