package somlib.som.descriptions;

import java.io.*;
import java.util.Date;
import java.text.DateFormat;

public class BookmarksVectorInDesc extends VectorInDesc {
/*
 * the specific type "html" of a Vector Description File
 */

  private java.util.Vector unknown = null;
  public String vec_id = null;
  public static String type = new String("bkmk");
  public String title = null;
  public String url = null;
  public String domain = null;
  public String time = null;
  public String lastmodified = null;
  public long size = -1;
  public int timesvisited = -1;
  public String lastvisited = null;
  public String description = null;

  public BookmarksVectorInDesc() {
    this.unknown = new java.util.Vector();
  }

  public BookmarksVectorInDesc(String filename) throws DescriptionException {
    File file = new File(filename);
    if((!file.exists()) || (!file.canRead()))
      throw new DescriptionException("Can't read file: "+file.toString());

    this.unknown = new java.util.Vector();
    readDescr(file);
  }

  public LibItem toLibItem(int xpos, int ypos, String imageLocation) throws DescriptionException
  /*
   * converts the email - description to a LibItem
   */
  {
    somlib.som.descriptions.LibItem libit = new somlib.som.descriptions.LibItem();

    libit.type=type;
    libit.title=title;
    libit.subject=title;
    libit.creator=vec_id;
    libit.identifier=url;

    libit.height=160;
    if(size>0) {
      libit.width= (int)((10*Math.log(size)) / 3);
      if(libit.width < 30)
        libit.width=30;
    } else libit.width = 40;

    int age = 0;
    try {
      age = (int)((new Date()).getTime() - DateFormat.getDateInstance().parse(lastmodified).getTime()) / (1000 * 60 * 60 *24); // age in days
    } catch (java.text.ParseException e) {
      System.out.println("Can't read Date "+lastmodified+": "+e.getMessage());
      age=6;
    }
    if(age<1) libit.dust_level=0;
    else if(age<2) libit.dust_level=1;
    else if(age<4) libit.dust_level=2;
    else if(age<7) libit.dust_level=3;
    else if(age<10) libit.dust_level=4;
    else if(age>15) libit.dust_level=5;

    libit.date=time;
    libit.size_byte=Long.toString(size);
    libit.highlighting=0;
    if(timesvisited > 0) libit.wellthumbed="paint wellthumbed texture";
    libit.template="paperback";

    libit.color_front=RGB.green();
    libit.color_top=RGB.violet();
    libit.color_spine=RGB.white();
    libit.color_title=RGB.black();
    libit.color_creator=RGB.black();

    libit.XPos = xpos;
    libit.YPos = ypos;
    libit.ZPos = (float)0.5;

    libit.description = description;
    if(imageLocation!=null)
      libit.image_logo = imageLocation+File.separator+domain;
    else
      libit.image_logo = domain;

    return libit;
  }

  public void writeDescr(String filename) throws DescriptionException {
    File file = new File(filename);

    try {
      BufferedWriter out = new BufferedWriter(new FileWriter(file));

      if(vec_id != null) {
        out.write("$VEC_ID "+vec_id); out.newLine();
      } else {
        out.write("$VEC_ID "+filename.substring(1+filename.lastIndexOf(File.separator)) ); out.newLine();
      }
      out.write("$TYPE "+type); out.newLine();
      if(title != null) {
        out.write("$TITLE "+title); out.newLine();
      }
      if(url != null) {
        out.write("$URL "+url); out.newLine();
      }
      if(domain != null) {
        out.write("$DOMAIN "+domain); out.newLine();
      }
      if(time != null) {
        out.write("$TIME "+time); out.newLine();
      }
      if(lastmodified != null) {
        out.write("$LASTMODIFIED "+lastmodified); out.newLine();
      }
      if(size != -1) {
        out.write("$SIZE "+size); out.newLine();
      }
      if(timesvisited >= 0) {
        out.write("$TIMESVISITED "+timesvisited); out.newLine();
      }
      if(lastvisited != null) {
        out.write("$LASTVISITED "+lastvisited); out.newLine();
      }
      if(description != null) {
        out.write("$DESCRIPTION "+description); out.newLine();
      }
      for (java.util.Enumeration enum = unknown.elements(); enum.hasMoreElements(); ) {
        out.write((String)enum.nextElement()); out.newLine();
      }
      out.close();
    } catch(IOException e) {
      throw new DescriptionException("File communication failure: "+e.getMessage());
    }
  }

  void readDescr(File source) throws DescriptionException {
    if(!source.exists() || !source.canRead())
      throw new DescriptionException("Couldn't find file: "+source.toString());

    String str, teil;
    int curvec=0;
    try {
      BufferedReader in = new BufferedReader(new FileReader(source));
      while ((str = in.readLine()) != null) {
        if(str.startsWith("$")) {
          teil=str.substring(1,str.indexOf(" "));

          if(teil.equals("VEC_ID"))
            vec_id=str.substring(str.indexOf(" ")+1);
          else if(teil.equals("TYPE"))
            type=str.substring(str.indexOf(" ")+1);
          else if(teil.equals("TITLE"))
            title=str.substring(str.indexOf(" ")+1);
          else if(teil.equals("URL"))
            url=str.substring(str.indexOf(" ")+1);
          else if(teil.equals("DOMAIN"))
            domain=str.substring(str.indexOf(" ")+1);
          else if(teil.equals("TIME"))
            time=str.substring(str.indexOf(" ")+1);
          else if(teil.equals("LASTMODIFIED"))
            lastmodified=str.substring(str.indexOf(" ")+1);
          else if(teil.equals("SIZE"))
            size=Long.parseLong(str.substring(str.indexOf(" ")+1));
          else if(teil.equals("TIMESVISITED"))
            timesvisited=Integer.parseInt(str.substring(str.indexOf(" ")+1));
          else if(teil.equals("LASTVISITED"))
            lastvisited=str.substring(str.indexOf(" ")+1);
          else if(teil.equals("DESCRIPTION"))
            description=str.substring(str.indexOf(" ")+1);
          else {
            unknown.add(str);
            System.out.println("Unknown Parameter: "+str);
          }

        } else if(!str.startsWith("#"))
          System.out.println("Read VectorInDesc: Wrong formatted file: "+source.toString());
        }
      in.close();
    } catch(IOException e) {
      throw new DescriptionException("Failure communicating with file: "+e.getMessage());
    }
  }

}
