package somlib.som.descriptions;

import java.io.*;
import java.util.Date;
import java.text.DateFormat;

public class EmailVectorInDesc extends VectorInDesc {
/*
 * the specific type "email" of a Vector Description File
 */

  private java.util.Vector unknown = null;
  public String vec_id = null;
  public static String type = new String("email");
  public String mailtype = null;
  public String subject = null;
  public String author = null;
  public String url = null;
  public String domain = null;
  public String description = null;
  public String date = null;
  public String time = null;
  public String timewritten = null;
  public long size = -1;
  public boolean  read = true;
  public int priority = 3;
      /* die priority skala ist wie folgt:
        X-Priority: 1 (Highest)
        X-Priority: 2 (High)
        X-Priority: 3 (Normal)   //ist default => wenn nicht vorhanden wird diese stufe angenommen
        X-Priority: 4 (Low)
        X-Priority: 5 (Lowest) */
  public boolean answered = false;
  public boolean attachment = false;

  public EmailVectorInDesc() {
    this.unknown = new java.util.Vector();
  }

  public EmailVectorInDesc(String filename) throws DescriptionException {
    File file = new File(filename);
    if((!file.exists()) || (!file.canRead()))
      throw new DescriptionException("Can't read file: "+file.toString());

    this.unknown = new java.util.Vector();
    readDescr(file);
  }

  public LibItem toLibItem(int xpos, int ypos, String imageLocation) throws DescriptionException
  /*
   * converts the email - description to a LibItem
   */
  {
    somlib.som.descriptions.LibItem libit = new somlib.som.descriptions.LibItem();

    libit.type=type;
    libit.format=mailtype;
    libit.title=subject;
    libit.subject=subject;
    libit.creator=author;
    libit.identifier=url;

    if(attachment) libit.height=165;
    else libit.height=155;
    if(size>0) {
      libit.width= (int)((10*Math.log(size)) / 2);
      if(libit.width < 30)
        libit.width=30;
    } else libit.width = 40;

    int age = 0;
    try {
      java.text.SimpleDateFormat formatter = new java.text.SimpleDateFormat ("dd.MM.yyyy");
      age = (int)((new Date()).getTime() - formatter.parse(date).getTime()) / (1000 * 60 * 60 *24); // age in days
    } catch (java.text.ParseException e) {
      System.out.println("Can't parse Date: "+date+": "+e.getMessage());
      age=6;
    }
    if(age<1) libit.dust_level=0;
    else if(age<2) libit.dust_level=1;
    else if(age<4) libit.dust_level=2;
    else if(age<7) libit.dust_level=3;
    else if(age<10) libit.dust_level=4;
    else if(age>15) libit.dust_level=5;

    libit.date=timewritten;
    libit.size_byte=Long.toString(size);
    if(read) libit.highlighting=0;
    else libit.highlighting=1;
    if(answered) libit.wellthumbed="paint wellthumbed texture";
    if(attachment) libit.template="folder";
    else libit.template="paperback";

    libit.color_front=RGB.blue();
    libit.color_top=RGB.violet();
    libit.color_spine=RGB.white();
    libit.color_title=RGB.black();
    libit.color_creator=RGB.black();

    libit.XPos = xpos;
    libit.YPos = ypos;
    libit.ZPos = (float)(priority-1)/4;

    libit.description = description;
    if(imageLocation!=null)
      libit.image_logo = imageLocation+File.separator+domain;
    else
      libit.image_logo = domain;

    return libit;
  }

  public void writeDescr(String filename) throws DescriptionException {
    File file = new File(filename);

    try {
      BufferedWriter out = new BufferedWriter(new FileWriter(file));

      if(vec_id != null) {
        out.write("$VEC_ID "+vec_id); out.newLine();
      } else {
        out.write("$VEC_ID "+filename.substring(1+filename.lastIndexOf(File.separator)) ); out.newLine();
      }
      out.write("$TYPE "+type); out.newLine();
      if(mailtype != null) {
        out.write("$MAILTYPE "+mailtype); out.newLine();
      }
      if(subject != null) {
        out.write("$SUBJECT "+subject); out.newLine();
      }
      if(author != null) {
        out.write("$AUTHOR "+author); out.newLine();
      }
      if(url != null) {
        out.write("$URL "+url); out.newLine();
      }
      if(description != null) {
        out.write("$DESCRIPTION "+description); out.newLine();
      }
      if(domain != null) {
        out.write("$DOMAIN "+domain); out.newLine();
      }
      if(date != null) {
        out.write("$DATE "+date); out.newLine();
      }
      if(time != null) {
        out.write("$TIME "+time); out.newLine();
      }
      if(timewritten != null) {
        out.write("$TIMEWRITTEN "+timewritten); out.newLine();
      }
      if(size != -1) {
        out.write("$SIZE "+size); out.newLine();
      }
      if(read) {
        out.write("$READ t");
        out.newLine();
      } else {
        out.write("$READ f");
        out.newLine();
      }
      if(priority != -1) {
        out.write("$PRIORITY "+priority); out.newLine();
      }
      if(answered) {
        out.write("$ANSWERED t");
        out.newLine();
      } else {
        out.write("$ANSWERED f");
        out.newLine();
      }
      if(attachment) {
        out.write("$ATTACHMENT t");
        out.newLine();
      } else {
        out.write("$ATTACHMENT f");
        out.newLine();
      }
      for (java.util.Enumeration enum = unknown.elements(); enum.hasMoreElements(); ) {
        out.write((String)enum.nextElement()); out.newLine();
      }
      out.close();
    } catch(IOException e) {
      throw new DescriptionException("File communication failure: "+e.getMessage());
    }
  }

  void readDescr(File source) throws DescriptionException {
    if(!source.exists() || !source.canRead())
      throw new DescriptionException("Couldn't find file: "+source.toString());

    String str, teil;
    int curvec=0;
    try {
      BufferedReader in = new BufferedReader(new FileReader(source));
      while ((str = in.readLine()) != null) {
        if(str.startsWith("$")) {
          teil=str.substring(1,str.indexOf(" "));

          if(teil.equals("VEC_ID"))
            vec_id=str.substring(str.indexOf(" ")+1);
          else if(teil.equals("TYPE"))
            type=str.substring(str.indexOf(" ")+1);
          else if(teil.equals("MAILTYPE"))
            mailtype=str.substring(str.indexOf(" ")+1);
          else if(teil.equals("SUBJECT"))
            subject=str.substring(str.indexOf(" ")+1);
          else if(teil.equals("AUTHOR"))
            author=str.substring(str.indexOf(" ")+1);
          else if(teil.equals("URL"))
            url=str.substring(str.indexOf(" ")+1);
          else if(teil.equals("DOMAIN"))
            domain=str.substring(str.indexOf(" ")+1);
          else if(teil.equals("DESCRIPTION"))
            description=str.substring(str.indexOf(" ")+1);
          else if(teil.equals("DATE"))
            date=str.substring(str.indexOf(" ")+1);
          else if(teil.equals("TIME"))
            time=str.substring(str.indexOf(" ")+1);
          else if(teil.equals("TIMEWRITTEN"))
            timewritten=str.substring(str.indexOf(" ")+1);
          else if(teil.equals("SIZE"))
            size=Long.parseLong(str.substring(str.indexOf(" ")+1));
          else if(teil.equals("READ"))
            read=str.charAt(str.indexOf(" ")+1)=='t';
          else if(teil.equals("PRIORITY"))
            priority=Integer.parseInt(str.substring(str.indexOf(" ")+1));
          else if(teil.equals("ANSWERED"))
            answered=str.charAt(str.indexOf(" ")+1)=='t';
          else if(teil.equals("ATTACHMENT"))
            attachment=str.charAt(str.indexOf(" ")+1)=='t';
          else {
            unknown.add(str);
            System.out.println("Unknown Parameter: "+str);
          }

        } else if(!str.startsWith("#"))
          System.out.println("Read VectorInDesc: Wrong formatted file: "+source.toString());
        }
      in.close();
    } catch(IOException e) {
      throw new DescriptionException("Failure communicating with file: "+e.getMessage());
    }
  }

}
