package somlib.som.descriptions;

import java.io.*;

public class Label {
/*
 * this class handles file communication for  SOM Label Files
 */

  /** Parameters:  **/
  public java.util.Vector items = null;
  private java.util.Vector unknown = null;

  public String type=new String("label");
  public int XDim=-1;
  public int YDim=-1;
  public int vec_dim=-1;
  public double min_weight=-1;
  public int max_labels=-1;

  public Label(int XDim, int YDim, int vec_dim, double min_weight, int max_labels) {
    this.XDim=XDim;
    this.YDim=YDim;
    this.vec_dim=vec_dim;
    this.min_weight=min_weight;
    this.max_labels=max_labels;
    items = new java.util.Vector();
    this.unknown = new java.util.Vector();
  }

  public Label(String filename) throws DescriptionException {
    File file = new File(filename);
    if((!file.exists()) || (!file.canRead()))
      throw new DescriptionException("Can't read file: "+file.toString());

    items = new java.util.Vector();
    this.unknown = new java.util.Vector();
    readDescr(file);
  }

  public String endung() {
    return ".lbl";
  }

  public void addItem(LabelItem lbi) {
    items.add(lbi);
  }

  public void addItems(java.util.Collection lbis) {
    items.addAll(lbis);
  }

  public void writeDescr(String filename) throws DescriptionException {
    LabelItem sorted[] = (LabelItem[])items.toArray(new LabelItem[0]);
    java.util.Arrays.sort(sorted);
    File file = new File(filename);

    try {
      BufferedWriter out = new BufferedWriter(new FileWriter(file));

      out.write("$TYPE "+type); out.newLine();
      out.write("$XDIM "+XDim); out.newLine();
      out.write("$YDIM "+YDim); out.newLine();
      out.write("$VEC_DIM "+vec_dim); out.newLine();
      if(min_weight>=0) {
        out.write("$MIN_WEIGHT "+min_weight); out.newLine();
      }
      if(max_labels>=0) {
        out.write("$MAX_LABELS "+max_labels); out.newLine();
      }
      for (java.util.Enumeration enum = unknown.elements(); enum.hasMoreElements(); ) {
        out.write((String)enum.nextElement()); out.newLine();
      }
      for(int i=0; i<sorted.length; i++) {
        sorted[i].writeDescr(out);
      }

      out.close();
    } catch(IOException e) {
      throw new DescriptionException("somlib.som.descriptions.Label.writeDescr(): File communication failure: "+e.getMessage());
    }catch(DescriptionException e) {
      throw new DescriptionException("somlib.som.descriptions.Label.writeDescr(): "+e.getMessage());
    }
  }

  private void readDescr(File source) throws DescriptionException {
    if(!source.exists())
      throw new DescriptionException("Couldn't find file: "+source.toString());

    String str, teil;
    try {
      BufferedReader in = new BufferedReader(new FileReader(source));
      while ((str = in.readLine()) != null) {
        if(str.startsWith("$")) {
          teil=str.substring(1,str.indexOf(" "));

          if(teil.equals("TYPE"))
            type=str.substring(str.indexOf(" ")+1);
          else if(teil.equals("XDIM"))
            XDim=Integer.parseInt(str.substring(str.indexOf(" ")+1));
          else if(teil.equals("YDIM"))
            YDim=Integer.parseInt(str.substring(str.indexOf(" ")+1));
          else if(teil.equals("VEC_DIM"))
            vec_dim=Integer.parseInt(str.substring(str.indexOf(" ")+1));
          else if(teil.equals("MIN_WEIGHT"))
            min_weight=Integer.parseInt(str.substring(str.indexOf(" ")+1));
          else if(teil.equals("MAX_LABELS"))
            max_labels=Integer.parseInt(str.substring(str.indexOf(" ")+1));
          else {
            unknown.add(str);
            System.out.println("Unknown Parameter: "+str);
          }
        } else if(!str.startsWith("#")) {
          items.add(new LabelItem(str) );
        }
      }
      in.close();
    } catch(IOException e) {
      throw new DescriptionException("Failure communicating with file: "+e.getMessage());
    }
  }

}