package somlib.som.descriptions;

import java.io.*;

public class Node {
/*
 * this class handles file communication for  SOM Unit Descriptions
 */

  /** Parameters:  **/
  public java.util.Vector items = null;
  private java.lang.StringBuffer comment = new java.lang.StringBuffer();
       //mandatory
  public StringParameter type=new StringParameter();
  public IntegerParameter XDim=new IntegerParameter();
  public IntegerParameter YDim=new IntegerParameter();

  public Node(String type, int XDim, int YDim) {
    this.type=new StringParameter(type, null);
    this.XDim=new IntegerParameter(XDim, null);
    this.YDim=new IntegerParameter(YDim, null);
    items = new java.util.Vector();
  }

  public Node(String filename) throws DescriptionException {
    File file = new File(filename);
    if((!file.exists()) || (!file.canRead()))
      throw new DescriptionException("Can't read file: "+file.toString());

    items = new java.util.Vector();
    readDescr(file);
  }

  public String endung() {
    return ".unit";
  }

  public void addItem(NodeItem ndi) {
    items.add(ndi);
  }

  public NodeItem getNodeAt(int x, int y) {
    if(items==null || items.isEmpty() || (y*XDim.parameter + x)>=items.size())
      return null;

    NodeItem zurueck;
    if( (zurueck=(NodeItem)items.elementAt(y*XDim.parameter + x)).hasCoordinates(x,y) )    /* fastest for sequential file */
      return zurueck;

    for (java.util.Enumeration enum = items.elements(); enum.hasMoreElements(); ) {   /* slow for random file */
      if( (zurueck=(NodeItem)enum.nextElement()).hasCoordinates(x,y) )
        return zurueck;
    }

    return null;
  }

  public void writeDescr(String filename) throws DescriptionException {
    File file = new File(filename);

    try {
      BufferedWriter out = new BufferedWriter(new FileWriter(file));
      out.write(comment.toString());

      out.write("$TYPE"+type.toString()); out.newLine();
      out.write("$XDIM"+XDim.toString()); out.newLine();
      out.write("$YDIM"+YDim.toString()); out.newLine();
      for (java.util.Enumeration enum = items.elements(); enum.hasMoreElements(); ) {
        ((NodeItem)enum.nextElement()).writeDescr(out);
        out.flush();
      }

      out.close();
    } catch(IOException e) {
      throw new DescriptionException("descriptions.Node.writeDescr(): File communication failure: "+e.getMessage());
    } catch(DescriptionException e) {
      throw new DescriptionException("descriptions.Node.writeDescr(): "+e.getMessage());
    }
  }

  private void readDescr(File source) throws DescriptionException {
    if(!source.exists()  || !source.canRead())
      throw new DescriptionException("Couldn't find file: "+source.toString());

    String str, teil;
    NodeItem ndi = null;
    try {
      BufferedReader in = new BufferedReader(new FileReader(source));
      while ((str = in.readLine()) != null && str.startsWith("#")) {
        comment.append(str+"\n");
      }
      while (str != null) {
        if(str.startsWith("$"))
          teil=str.substring(1,str.indexOf(" "));
        else teil=str;   //just anything but TYPE, XDIM or YDIM

        if(teil.equals("TYPE"))
          type=new StringParameter(str.substring(str.indexOf(" ")+1));
        else if(teil.equals("XDIM"))
          XDim=new IntegerParameter(str.substring(str.indexOf(" ")+1));
        else if(teil.equals("YDIM"))
          YDim=new IntegerParameter(str.substring(str.indexOf(" ")+1));
        else {

          do {
            if(ndi==null) {
              ndi = new NodeItem();
              while(str!=null && str.startsWith("#")) {
                ndi.comment.append(str+"\n");
                str=in.readLine();
              }
            }
            if(str.startsWith("$")) {
              teil=str.substring(1,str.indexOf(" "));
              if(teil.equals("POS_X")) {
                if(ndi.pos_x.parameter == -1)
                  ndi.pos_x=new IntegerParameter(str.substring(str.indexOf(" ")+1));
                else {
                  items.add(ndi); ndi = null; continue;
                }
              } else if(teil.equals("POS_Y")) {
                if(ndi.pos_y.parameter == -1)
                  ndi.pos_y=new IntegerParameter(str.substring(str.indexOf(" ")+1));
                else {
                  items.add(ndi); ndi = null; continue;
                }
              } else if(teil.equals("UNIT_ID")) {
                if(ndi.unit_id.parameter == null)
                  ndi.unit_id=new StringParameter(str.substring(str.indexOf(" ")+1));
                else {
                  items.add(ndi); ndi = null; continue;
                }
              } else if(teil.equals("QUANTERROR_UNIT")) {
                if(ndi.quanterror_unit.parameter == -1)
                  ndi.quanterror_unit=new FloatParameter(str.substring(str.indexOf(" ")+1));
                else {
                  items.add(ndi); ndi = null; continue;
                }
              } else if(teil.equals("QUANTERROR_UNIT_AVG")) {
                if(ndi.quanterror_unit_avg.parameter == -1)
                  ndi.quanterror_unit_avg=new FloatParameter(str.substring(str.indexOf(" ")+1));
                else {
                  items.add(ndi); ndi = null; continue;
                }
              } else if(teil.equals("UMAT_UNIT")) {
                if(ndi.umat_unit.parameter == -1)
                  ndi.umat_unit=new FloatParameter(str.substring(str.indexOf(" ")+1));
                else {
                  items.add(ndi); ndi = null; continue;
                }
              } else if(teil.equals("UMAT_RIGHT")) {
                if(ndi.umat_right.parameter == -1)
                  ndi.umat_right=new FloatParameter(str.substring(str.indexOf(" ")+1));
                else {
                  items.add(ndi); ndi = null; continue;
                }
              } else if(teil.equals("UMAT_DOWN")) {
                if(ndi.umat_down.parameter == -1)
                  ndi.umat_down=new FloatParameter(str.substring(str.indexOf(" ")+1));
                else {
                  items.add(ndi); ndi = null; continue;
                }
              } else if(teil.equals("UMAT_DOWN_LEFT")) {
                if(ndi.umat_down_left.parameter == -1)
                  ndi.umat_down_left=new FloatParameter(str.substring(str.indexOf(" ")+1));
                else {
                  items.add(ndi); ndi = null; continue;
                }
              } else if(teil.equals("UMAT_DOWN_RIGHT")) {
                if(ndi.umat_down_right.parameter == -1)
                  ndi.umat_down_right=new FloatParameter(str.substring(str.indexOf(" ")+1));
                else {
                  items.add(ndi); ndi = null; continue;
                }
              } else if(teil.equals("NR_VEC_MAPPED")) {
                if(ndi.nr_vec_mapped.parameter == -1)
                  ndi.nr_vec_mapped=new IntegerParameter(str.substring(str.indexOf(" ")+1));
                else {
                  items.add(ndi); ndi = null; continue;
                }
              } else if(teil.equals("MAPPED_VECS")) {
                if(ndi.mapped_vecs.parameter == null)
                  ndi.mapped_vecs=new StringParameter(str.substring(str.indexOf(" ")+1));
                else {
                  items.add(ndi); ndi = null; continue;
                }
              } else if(teil.equals("MAPPED_VECS_DIST")) {
                if(ndi.mapped_vecs_dist.parameter == null)
                  ndi.mapped_vecs_dist=new ListFloatParameter(str.substring(str.indexOf(" ")+1));
                else {
                  items.add(ndi); ndi = null; continue;
                }
  /*             } else if(teil.equals("NR_SOMS_MAPPED")) {
                  nr_soms_mapped=(new Integer(str.substring(str.indexOf(" ")+1))).intValue();
                } else if(teil.equals("URL_MAPPED_SOMS")) {
                  url_mapped_soms=str.substring(str.indexOf(" ")+1);  */
                } else if(teil.equals("NR_UNIT_LABELS")) {
                if(ndi.nr_unit_labels.parameter == -1)
                  ndi.nr_unit_labels=new IntegerParameter(str.substring(str.indexOf(" ")+1));
                else {
                  items.add(ndi); ndi = null; continue;
                }
              } else if(teil.equals("UNIT_LABELS")) {
                if(ndi.unit_labels.parameter == null)
                  ndi.unit_labels=new ListStringParameter(str.substring(str.indexOf(" ")+1));
                else {
                  items.add(ndi); ndi = null; continue;
                }
              } else if(teil.equals("UNIT_LABELS_QE")) {
                if(ndi.unit_labels_qe.parameter == null)
                  ndi.unit_labels_qe=new ListFloatParameter(str.substring(str.indexOf(" ")+1));
                else {
                  items.add(ndi); ndi = null; continue;
                }
              } else if(teil.equals("UNIT_LABELS_WGT")) {
                if(ndi.unit_labels_wgt.parameter  == null)
                  ndi.unit_labels_wgt=new ListFloatParameter(str.substring(str.indexOf(" ")+1));
                else {
                  items.add(ndi); ndi = null; continue;
                }
              } else if(teil.equals("DESCRIPTION")) {
                if(ndi.description.parameter == null)
                  ndi.description=new StringParameter(str.substring(str.indexOf(" ")+1));
                else {
                  items.add(ndi); ndi = null; continue;
                }
              } else {
                ndi.unknown.add(str);
                System.out.println("Unknown Parameter: "+str);
              }

            } else if(str.startsWith("#")) {
              items.add(ndi);
              ndi=null;
              continue;
            } else
              System.out.println("Wrong formatted file: "+str);

            str = in.readLine();
          } while(str != null);
          items.add(ndi);
        }
        str = in.readLine();
      }
      in.close();
    } catch(IOException e) {
      throw new DescriptionException("Failure communicating with file: "+e.getMessage());
    }
  }

}
