package somlib.som.descriptions;

import java.io.*;

public abstract class VectorInDesc {
/*
 * all types of Vector Description files have to obey this format
 */

  public static VectorInDesc fromType(String filename) throws DescriptionException {
    if(!(new File(filename)).exists())
      return null;

    String type = VectorInDesc.extractType(filename);
    if(type.compareTo("email")==0)                                      // new types ought to be added here
      return new EmailVectorInDesc(filename);
    else if(type.compareTo("html")==0)
      return new BookmarksVectorInDesc(filename);

    System.out.println("VectorInDesc.fromType(String): VectorIn-Type "+type+" not known in file: "+filename);
    return null;
  }

  public static String extractType(String from) throws DescriptionException {
    try {
      BufferedReader in = new BufferedReader(new FileReader(from));
      String str;
      while ((str = in.readLine()) != null) {
        if(str.startsWith("$TYPE"))
          return str.substring(str.indexOf(" ")+1);
      }
    } catch(IOException e) {
      throw new DescriptionException("VectorInDesc.fromType(String): "+e.getMessage());
    }
    throw new DescriptionException("VectorInDesc.fromType(String): Type couldn't be extracted.");
  }

  public static String endung()
  /*
   * this one is not unchangeable for a specific type, but it's by default this way
   */
  {
    return ".vec";
  }

  public static LibItem standardLibItem(int xpos, int ypos, String name, String imageLocation) throws DescriptionException
  /*
   * produces a standard LibItem
   */
  {
    somlib.som.descriptions.LibItem libit = new somlib.som.descriptions.LibItem();

    libit.type="empty";
    libit.format="empty";
    libit.title=name;
    libit.subject=name;
    libit.creator="unknown";
    libit.identifier="unknown";
    libit.height = 160;
    libit.width = 40;
    libit.dust_level=0;

    java.util.Date currentTime = new java.util.Date();
    java.text.SimpleDateFormat formatter = new java.text.SimpleDateFormat ("dd.MM.yyyy, hh:mm:ss");
    libit.date=formatter.format(currentTime);
    libit.size_byte="0";
    libit.highlighting=0;
//    libit.wellthumbed="paint wellthumbed texture";
    libit.template="paperback";

    libit.color_front=RGB.blue();
    libit.color_top=RGB.violet();
    libit.color_spine=RGB.white();
    libit.color_title=RGB.black();
    libit.color_creator=RGB.black();

    libit.XPos = xpos;
    libit.YPos = ypos;
    libit.ZPos = (float)0.5;

    libit.description = null;
    if(imageLocation!=null)
      libit.image_logo = imageLocation+File.separator+"unknown";
    else
      libit.image_logo = "unknown";

    return libit;
  }

  abstract void readDescr(File source) throws DescriptionException;
  public abstract void writeDescr(String filename) throws DescriptionException;
  public abstract LibItem toLibItem(int xpos, int ypos, String imageLocation) throws DescriptionException;

}
