package somlib.som.functions;

import java.lang.*;

public class Functions {
/*
 * here the Neighborhood- and the Learnrate-Function are coordinated
 * and handed back to the Training
 */
  private somlib.textrepresentation.Logfile log = null;
  private byte verbose = 2;

  private Learnrate learn;
  private Neighborhood neighbor;

  public Functions(int XDim, int YDim, float eps0, String learn, float delta0, String neighbor, somlib.textrepresentation.Logfile log, byte verbose) throws FunctionsException
  /*
   * the Functions to be used are generated dynamically;
   * the right one is chosen by the class-name, given by @learn resp. @neighbor;
   * @eps0 and @delta0 are the initial values for the functions;
   * @XDim an @YDim describe the dimensions of the map;
   */
  {
    this.log=log;
    this.verbose=verbose;

    try {
      Class classlearn = Class.forName("somlib.som.functions."+learn);
      Class classneighbor = Class.forName("somlib.som.functions."+neighbor);
      this.learn = (Learnrate) (classlearn.newInstance());
      this.learn.define(XDim, YDim, eps0);
      this.neighbor = (Neighborhood) (classneighbor.newInstance());
      this.neighbor.define(XDim, YDim, delta0);
    } catch(Exception e) {
      throw new FunctionsException("Couldn't load Functions: "+e.getMessage());
    }
    if(log!=null)
      log.append("Start: Functions defined.");
    if (verbose > 1) System.out.println("    Functions defined.");
  }

  public float eps(int t) {
    return learn.eps(t);
  }

  public float hci(int t, float distance) {
    return neighbor.hci(t, distance);
  }

  public String learnrateType() {
    return learn.learnrateType();
  }

  public float learnrateInit() {
    return learn.learnrateInit();
  }

  public String neighborhoodType() {
    return neighbor.neighborhoodType();
  }

  public float neighborhoodInit() {
    return neighbor.neighborhoodInit();
  }

}
