package somlib.som.tools.transpose;

/**
 * Title:                                         SomLib
 * Description:Java Tools for the SomLib Project
 * Copyright:                                     Copyright (c) 2000
 * Company:                                       IFS
 * @author elan
 * @version 1.0
 */

import java.io.*;
import somlib.som.descriptions.*; // VectorIn
import somlib.som.labelling.*; // WordfileReader (Template File Reader)

public class Transpose {

/* param descr:
  -i indiv vector file in
  -o indiv vector file out
  -t template vector in
  -e template vector out
  -v verbose
  HINT: increase max heap size if out of memory exception (-Xmx50000000...)
  */
  public static void main(String[] args) throws TransposeException {
    if (args.length%2 == 1)
      usage();
    else {
      int param=0;
      String in_indiv=null, out_indiv=null;
      String in_template=null, out_template=null;
      int verbose= 1; // default
      boolean normalize = false;

      while(param < args.length) {      //getting the command-line parameters
        if((args[param].length()>2) || (args[param].charAt(0)!='-'))
          usage();

        switch(args[param].charAt(1)) {
          case 'i':
            in_indiv = new String(args[++param]);
            break;
          case 'o':
            out_indiv = new String(args[++param]);
            break;
          case 't':
            in_template = new String(args[++param]);
            break;
          case 'e':
            out_template = new String(args[++param]);
            break;
          case 'v':
            verbose = Integer.parseInt(args[++param]);
            break;
          default:
            usage();
        }
        param++;
      }

      if ((in_indiv==null)||(in_template==null)||(out_indiv==null)||(out_template==null)) usage();

      Transpose.write(out_indiv, out_template, in_template, in_indiv, (byte) verbose);
    }
  }

  private static void usage() {
    System.out.println("Usage: java somlib.som.transpose.Transpose -i <SOM-Input-Vector_in> -o <Transposed-SOM-Input-Vector_out> -t <SOM-Template-Vector_in> -e <Transposed-SOM-Template-Vector_out> [-v <verbosity>]");
    System.exit(1);
  }

  public static void write(String indiv_outfile, String template_outfile, String templateFile, String indivFile, byte verbose) throws TransposeException {
    if (verbose > 0) System.out.println("  start transposed ...");
    try {
      String[] words = WordfileReader.loadWords(templateFile, verbose);
      if (verbose > 1) System.out.println("    reading individualvecs (slowly) ...");
      VectorIn indiv = new VectorIn(indivFile);

      FakeTemplateVectorFile.write(indiv.labels, template_outfile, verbose);

      indiv.labels=words;
      indiv.vec_dim=indiv.XDim;
      indiv.XDim=words.length;

      if (verbose > 1) System.out.println("    creating big matrix: "+indiv.XDim+" x "+indiv.vec_dim+" ...");
      float[][] copy_vecs = new float[indiv.XDim][indiv.vec_dim];

      if (verbose > 1) System.out.println("     start transposing matrix of size: "+indiv.XDim+" x "+indiv.vec_dim+" ...");

      for (int i=0;i<indiv.XDim;i++)
        for (int j=0;j<indiv.vec_dim;j++)
          copy_vecs[i][j]=indiv.vecs[j][i];

      indiv.vecs=copy_vecs;
      if (verbose > 1) System.out.println("     have written transposed template start writing indivdualvectors file...");

      indiv.writeDescr(indiv_outfile);

      if (verbose > 0) System.out.println("  done with transposed.");
    } catch (Exception e) {System.out.println("ERROR in TransposeIndividuals: "+ e.getMessage()); e.printStackTrace();}
  }
}