package somlib.textrepresentation.extractor;

import java.io.*;
import java.util.*;
import java.lang.*;
import somlib.textrepresentation.*;

/**
 *
 * @author elan
 * @version 1.2 (elias, trying to reduce output file size)
 */
public abstract class Extractor extends Object {

/**
 *  @param matrix file in which the result is stored
 *  @param inFiles individual wordlists that are used to create matrix
 *  @param tv template vector
 *  @param verbose extra information is given
 *  @exception PreprocessException if any error occurs
 */
  public void extract(File matrix, String type, File[] inFiles, Hashtable tv, byte verbose, Logfile l) throws TextrepresentationException {
    FileInputStream istream;
    ObjectInputStream in;
    Hashtable idv;

    if (verbose>0) System.out.println("  Extractor: start");
    if (tv != null) {
      try {
        // notwendig um verzeichnis anzulegen falls nicht existiert.
        int pfad;
        if((pfad = matrix.toString().lastIndexOf(File.separator)) > 0) {
          File dirOfmatrix = new File(matrix.toString().substring(0,matrix.toString().lastIndexOf(File.separator)));
          dirOfmatrix.mkdirs();
        }
        matrix.createNewFile();
        BufferedWriter out = new BufferedWriter(new FileWriter(matrix));

        out.write("$TYPE "+type); out.newLine();
        out.write("$XDIM " + ((Integer) tv.get("$files")).intValue()); out.newLine();
        out.write("$YDIM 1"); out.newLine();
        out.write("$VEC_DIM " + (tv.size()-1)); out.newLine();

        File template = new File(matrix.toString().substring(0,matrix.toString().lastIndexOf('.')));
        if (!template.exists())
          somlib.textrepresentation.TemplateVector.toTextFile(template.toString()+".tv", tv);

        if (verbose>2) System.out.println("      Individual Vectors: "+inFiles.length);
        for (int i = 0; i < inFiles.length; i++) {
          if (inFiles[i].isFile()) {
            if (verbose>2) System.out.println("      "+inFiles[i]);

            istream = new FileInputStream(inFiles[i]);
            in = new ObjectInputStream(istream);
            idv = (Hashtable)in.readObject();
            for (Enumeration enum = tv.keys(); enum.hasMoreElements(); ) {
              String key = (String) enum.nextElement();
              if (key.compareTo("$files")!=0) {
                if (idv.containsKey(key)) {
                  int myInt1 = ((Integer) idv.get(key)).intValue();
                  int myInt2 = ((TemplateItem) tv.get(key)).df;
                  out.write(((float) calculate(myInt1,myInt2,inFiles.length)) + " ");
                }
//                else {out.write(empty() + " ");} changed by elias
                else {out.write("0 ");}
              }
            }
            in.close();
            int j;
            if ((j = inFiles[i].toString().indexOf('.')) != -1) {j = inFiles[i].toString().length();} // nur falls eine der input dateien keinen punkt in namen hat
            out.write(inFiles[i].toString().substring(0,j)); // der filename ist die ID des idtf vectors
            out.newLine();
          } // kein verzeichnis
        } // gehe alle files durch
        out.flush(); out.close();
      } catch (Exception e) {throw new TextrepresentationException("Extractor: " + e.getMessage());}
    } // wenn tf ungleich null
    if (verbose>0) System.out.println("  Extractor: done.");
  } // extract


/**
 * @return value to be placed in matrix if the word is NOT in the file
 */
  public abstract double empty();

/**
 * @return value to be placed in matrix if the word is in the file
 */
  public abstract double calculate(int wordInFile, int filesWithWord, int totalFiles);

}
