package somlib.textrepresentation.parser;

import java.io.*;
import java.lang.*;
import java.util.Hashtable;
import somlib.textrepresentation.*;

/**
 * Any characters except for letters are delimiters, the letters inbetween are the words.
 *
 * @author elan
 * @version 1.1 (elias bzgl segmenation fault, ohne stringbuffer, streams werden jetzt 'genauer' geschlossen)
 */
public class Words extends Object implements Parser{
  private int minWordLength;

/**
 * @param minWordLength words with less then are ignored
 */
  public Words(int minWordLength) {
    this.minWordLength = minWordLength;
  } // constructor

/**
 * Does all the work.
 *
 * @param files to be used to create the individual vectors, the prefix of the files will be used again for the the individual vectors
 * @param outDir where the individual vectors are to be stored
 * @param verbose extra information is given when set to true
 * @param override determines if files should be replaced or skipped
 * @param fileSuffix of the output files
*/
  public File[] createFiles (File[] files ,String outDir, byte verbose, Logfile log, boolean override, String fileSuffix) throws TextrepresentationException
  {
    if (files == null) {throw new TextrepresentationException("Words: createFiles: filelist not defined.");}
    if (outDir == null) {throw new TextrepresentationException("Words: createFiles: output directory not defined."); }

    String myLine = null; // readLine aud input datei wird dort gespeichert
    File[] modifiedFiles = new File[files.length];
    int index = 0;
    Hashtable idtf  = new Hashtable();
    File dirOutF = null;
    String str = null;
    Integer n = null;

    if (verbose>0) System.out.println("Start Words:");
    try {
      File outFile;
      BufferedReader in;
      ObjectOutputStream out;
      dirOutF = new File(outDir);
      if (dirOutF.mkdirs()) {if (verbose>1) System.out.println("Directory created: "+dirOutF.toString());}

    if (verbose>1) System.out.println("  Files: "+files.length);

      for (int i = 0; i < files.length; i++) {
        if (!files[i].isDirectory()) {
          // output datei anlegen
          outFile = new File(outDir + dirOutF.separator +files[i].getName() + fileSuffix);
          if (override || outFile.createNewFile())
          {
            if (verbose>2) System.out.println("    "+files[i].toString());

            modifiedFiles[index] = outFile; index++;
            FileReader istream = new FileReader(files[i]);
            in = new BufferedReader(istream);
            FileOutputStream ostream = new FileOutputStream(outFile); 
            out = new ObjectOutputStream(ostream);

            idtf.clear();
            int k=0;
            int wordLength=0;
            int start=0;
            while ((myLine = in.readLine()) != null) {
              for (k=0; k<myLine.length(); k++) {
                if  ((Character.isLetter(myLine.charAt(k)))) wordLength++;
                if  (!(Character.isLetter(myLine.charAt(k))) || (k == myLine.length()-1)) {
                  if (wordLength >= minWordLength) { // wort lang genug. 
                    str = (new String(myLine.substring(start,start+wordLength))).toLowerCase();
                    n = (Integer) idtf.get(str);
                    if (n != null) {
                      idtf.remove(str); 
                      idtf.put(str, new Integer(n.intValue()+1)); // erhoehe den zaehler um eins
                    }
                    else {idtf.put(str, new Integer(1));}
                  }
                  if (k == myLine.length()-1) start=0; else start=k+1; 
                  wordLength=0;
                }// trage tmpWord in liste ein und reset tmpWord
              } // gehe zeile durch
            } // solange nicht EOF
            out.writeObject(idtf);
            out.flush(); ostream.close(); out.close();
            istream.close(); in.close();
/*long freevorher=Runtime.getRuntime().freeMemory();
long nichtfreivor=Runtime.getRuntime().totalMemory();
System.gc();
long freenachher=Runtime.getRuntime().freeMemory();
long nichtfreinach=Runtime.getRuntime().totalMemory();
System.out.println("Free:\t" + freevorher+"\t"+freenachher + "not Free:\t" +nichtfreivor +"\t"+nichtfreinach);
*/
          } // file existiert schon -> ignorieren, mit naechstem weitermachen
        }
      } // gehe alle dateien in der liste durch

      File[] modifiedFilesCut = new File[index];
      for (int i = 0; i<index; i++) modifiedFilesCut[i] = modifiedFiles[i];
      if (verbose>0) System.out.println("Done.");
      return modifiedFilesCut;
    } catch (Exception e) {throw new TextrepresentationException("Error in Words: createFiles: " + e.getMessage());}
  } // createFiles

} // class words
