package somlib.textrepresentation.preprocess.bookmarks;

import java.util.*;
import java.io.*;
import java.lang.*;
import somlib.textrepresentation.*;

public class VecDescrWriter {

  private BufferedReader page=null;
  private BufferedWriter dest = null;
  private String filename=null;
  private Logfile log_;
  private byte verbose;

    /** only for testing reasons */
  public static void main(String[] args) throws BookmarksException {
    new VecDescrWriter((byte)4, null, 43, "E:"+File.separator+"testbk"+File.separator+"web"+File.separator+"Aboutthi.htm", "tested");
  }

  public VecDescrWriter(byte verbose, Logfile l, long date, String input, String output) throws BookmarksException {
    log_=l;
    this.verbose=verbose;
    this.filename=output;

    if (verbose > 2)
      System.out.println("      VecDescrWriter: writing Vector-Description to "+output);

    try {
      page=new BufferedReader(new FileReader(input));
      dest = new BufferedWriter(new FileWriter(output, false));   //ueberschreibt vorhandenes file
    } catch(FileNotFoundException e) {
      if (verbose > 1) System.out.println("Internal Fault getting the correct names of the pages: "+e.getMessage());
      throw new BookmarksException("VecDescrWriter: Internal Fault getting the correct names of the pages: "+e.getMessage());
    } catch(IOException e) {
      if (verbose > 1) System.out.println("Couldn't create Vector-Description-File: "+e.getMessage());
      throw new BookmarksException("VecDescrWriter: Couldn't create Vector-Description-File: "+e.getMessage());
    }

    writeInfo((new File(input)).length(), date);

    try {
      dest.close();
      page.close();
    } catch(IOException e) {
      if (verbose > 1) System.out.println("Couldn't close input: "+e.getMessage());
      throw new BookmarksException("VecDescrWriter: Couldn't close input: "+e.getMessage());
    }

    if (verbose > 2)
      System.out.println("      VecDescrWriter: successfully finished writing.");
  }


  private void writeInfo(long size, long lastmodified) throws BookmarksException {
    int a, pos;
    String str, title=null, url=null;
    try {
      while((str=page.readLine()) != null) {
        if((pos = str.toLowerCase().indexOf("<title>")) >= 0) {
          if((a=str.indexOf('<',pos+1)) < 0) {
            title = page.readLine();
            if((a=title.indexOf('<')) > 0)
              title = title.substring(0,a);
          } else
            title = str.substring(str.indexOf('>', pos)+1, a);
          title.trim();
        }
        if(str.startsWith("<!URL "))
          url = str.substring(6, str.length() -1);
      }
    } catch(IOException e) {
      try {
        dest.close();
        page.close();
      } catch(IOException ex) {}
      if (verbose > 1) System.out.println("Couldn't read input: "+e.getMessage());
      throw new BookmarksException("VecDescrWriter: Couldn't read input: "+e.getMessage());
    }

    java.text.DateFormat df = java.text.DateFormat.getDateTimeInstance();
    Date date = new Date();
    Date modif = new Date(lastmodified);
    try {
      dest.write("$VEC_ID "+filename); dest.newLine();
      dest.write("$TYPE html"); dest.newLine();
      if(title!=null)
        dest.write("$TITLE "+title); dest.newLine();
      if(url!=null) {
        dest.write("$URL "+url); dest.newLine();
        int i, end = url.indexOf("/",url.indexOf("."));
        if(end<=0) end=url.length();
        for(i = end-1; (i>0) && (url.charAt(i) != '.'); i--) {}
        dest.write("$DOMAIN "+ url.substring(i+1, end)); dest.newLine();
      }
      dest.write("$TIME "+df.format(date)); dest.newLine();
      dest.write("$LASTMODIFIED "+df.format(modif)); dest.newLine();
      dest.write("$SIZE "+size); dest.newLine();
      dest.write("$TIMESVISITED 0"); dest.newLine();
//      dest.write("$LASTVISITED "); dest.newLine();
//      dest.write("$DESCRIPTION "); dest.newLine();
    } catch(IOException e) {
      try {
        dest.close();
        page.close();
      } catch(IOException ex) {}
      if (verbose > 1) System.out.println("Couldn't write descr: "+e.getMessage());
      throw new BookmarksException("VecDescrWriter: Couldn't write descr: "+e.getMessage());
    }
  }

}