package somlib.textrepresentation.preprocess.email;

import java.io.*;
import java.lang.*;
import somlib.textrepresentation.preprocess.email.contentType.*;
import somlib.textrepresentation.*;

/**
*
* @author Elias
* @version 1.0 (19.08.1999)
*/
public class EMails extends Object {

  private File[] emails;

/**
 *  This Class is used to filter and convert emails to plain text files.
 *  @param per email one file
 */
  public EMails(File[] emails) {
    this.emails=emails;
  } // constructor

/**
 *  with every email it checks if it has the desired suffix and then converts it
 *  @param inSuffix suffix of files in the input file list for example "1234.email" where ".email" is the suffix
 *  @param outSuffix suffix of created files for example "1234.plain" where ".plain" is the suffix
 *  @param outDir where to store created files
 *  @return List of Files which were created.
 *  @exception EMailException is thrown in any error case (file read/write, ..)
 *   vector description files will be stored in the destination-directory (@descrDir) with suffix (@vecSuffix)
 */
  public File[] extractPlainBodies(byte verbose, Logfile l, String inSuffix, String outSuffix, String vecSuffix, String outDir, String descrDir) throws EMailException {

    File[] modifiedFiles = new File[emails.length];
    File dirOutput = new File(outDir);
    int index = 0;

    if (verbose > 0) System.out.println("  EMails: Creating files for parser..."); // debug
    if (l!=null) {
      l.append("Start: EMails Module.");
      l.append("  Params:");
      l.append("    inSuffix: "+inSuffix);
      l.append("    outSuffix: "+outSuffix);
      l.append("    outDir: "+outDir);
      l.append("    descrDir: "+descrDir);
      for (int i=0; i < emails.length; i++) {
        l.append("    "+ (i+1)+" infile: "+emails[i]);
      }
      l.append("  Params end.");
    }

    if (emails == null) {throw new EMailException("EMails: empty File List!");}

    try { // verzeichnis: falls nicht vorhanden wird neues angelegt
      if (dirOutput.mkdirs()) {if (verbose > 1) System.out.println("    Verzeichnis angelegt: " + dirOutput.toString() + ".");}
    } catch (Exception e) {throw new EMailException("EMails: Can not mkdir: " + dirOutput.toString() +": "+ e.getMessage());}

    int i=0;
    try {
      int j = 0;
      for (i=0; i < emails.length; i++) { // gehe alle files durch
        if (!emails[i].isDirectory()) {
        // nur die mit entsprechender endung betrachten und die noch nicht gemacht wurden (xxx.plain existiert noch ncith):
          if (emails[i].length() > inSuffix.length()) {
            String simply = emails[i].toString();
            File outFile = new File(outDir + File.separator + simply.substring(simply.lastIndexOf(File.separator),simply.length()-inSuffix.length()) + outSuffix);
            if ((simply.substring(simply.length()-inSuffix.length(), simply.length()).compareToIgnoreCase(inSuffix) == 0) && (outFile.createNewFile())) {
              try {
                if (verbose > 2) System.out.println("      EMails File: " + emails[i].toString()); // debug

                EMailHeader h = new EMailHeader(emails[i]);
                if (h.contentType() != null) {
                  modifiedFiles[index] = outFile; index++;
                  if(descrDir != null) {
                    long size = h.contentType().getBodyText(emails[i], outFile, h.subject());
                    if(!descrDir.endsWith(File.separator))
                      descrDir = descrDir+File.separator;
                    new VecDescrWriter(verbose, l, h, size, descrDir + File.separator + simply.substring(simply.lastIndexOf(File.separator)+1,simply.length()-inSuffix.length()) + vecSuffix);
                  }
                }
                else {
                  j++; if (verbose>1) {System.out.println("    " + emails[i] + ": unknown type: " + h.helpContentType);}
                  outFile.delete(); // dieser typ kann nicht erkannt werden
                  if(l!=null)
                    l.append("    " + emails[i] + ": unknown type: " + h.helpContentType);
                }
              } catch (EMailException e) {if (verbose >1) System.out.println("    " + emails[i] + ": problems: "+ e.getMessage()); outFile.delete();}
            }
          }
        } // kein verzeichnis
      } // alle files durchgehen
      if (verbose > 1) System.out.println("    EMails with unknown types: " + j + "."); // debug
    } catch (Exception e) {
       if (verbose > 1) {
         System.out.println("Problem with email: "+emails[i]);
         System.out.println("Email number: "+i);
         System.out.println("Filelist.length: "+emails.length);
       }
       throw new EMailException("EMails: " + e.getMessage() +  ".");
    }

    File[] modifiedFilesCut = new File[index];
    for (i=0;i<index;i++) {modifiedFilesCut[i] = modifiedFiles[i];} //liste kuerzen (es kann sein, dass manche dateien nicht erstellt wurden..
    if (verbose > 0) System.out.println("  EMails: end.");
    if(l!=null)
      l.append("End: EMails Module.");
    return modifiedFilesCut;
  } // extractPlainBodies

} // class EMails

